unit frmMain;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, Messages, Forms, Controls, Dialogs, ExtCtrls, StdCtrls, LMessages,
  Buttons, Menus, uCEFTypes, uCEFInterfaces, uCEFWorkScheduler,
  uCEFBrowserWindow, uCEFChromium, Classes, uCEFChromiumEvents;

type

  { TMainForm }

  TMainForm = class(TForm)
    MLog: TMemo;
    miHideDevTools: TMenuItem;
    miSHowDevTools: TMenuItem;
    bwBrowser: TBrowserWindow;
    PMTray: TPopupMenu;
    splDevtools: TSplitter;
    TIApp: TTrayIcon;
    procedure btnBackClick(Sender: TObject);
    procedure ChromiumTitleChange(Sender: TObject; const browser: ICefBrowser;
      const title: ustring);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure bwBrowserBrowserClosed(Sender: TObject);
    procedure bwBrowserBrowserCreated(Sender: TObject);
  private
    FLogMsg : String;
    FNewTitle: string;
    procedure DoLog(Const aMsg : String);
    procedure DoLog(const aFmt : String; aArgs : array of const);
    procedure AddTrayMenuItem(aAPIID: integer; ACaption: String);
    procedure DoSetHTMLTitle(Data: PtrInt);
    procedure HandleProcessMessage(Sender: TObject; const browser: ICefBrowser;
      const frame: ICefFrame; sourceProcess: TCefProcessId;
      const aMessage: ICefProcessMessage; out Result: Boolean);
    procedure HandleTrayIconClick(Sender: TObject);
    procedure RemoveTrayMenuItem(aAPIID: integer);
    procedure SetTrayIconVisibility(aVisible: PtrInt);
    procedure ShowLogMsg(Data: PtrInt);
  protected
    {$IFDEF WINDOWS}
    procedure WMEnterMenuLoop(var aMessage: TMessage); message WM_ENTERMENULOOP;
    procedure WMExitMenuLoop(var aMessage: TMessage); message WM_EXITMENULOOP;
    {$ENDIF}
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

uses
   uCEFProcessMessage, CEFControl, embeddedapi, localresources;

Const
//  STrayIconURL = 'local:///html/trayicon.html';
  STrayIconURL = 'local:///html/trayicon-pas2js.html';


Type
  TTrayMenuItem = Class(TMenuItem)
  Public
    APIID : Integer;
  End;

{ TMainForm }

procedure TMainForm.HandleTrayIconClick(Sender: TObject);

var
  TempMsg    : ICefProcessMessage;
  MID : Integer;
begin
  MID:=(Sender as TTRayMenuItem).APIID;
  DoLog('BROWSER process: click for tray menu item %d',[MID]);
  // Send a message to the render process so it will execute the javascript handler associated with this menu item.
  TempMsg:=TCefProcessMessageRef.New(SMsgExecuteTrayMenuClick);
  if TempMsg.ArgumentList.SetInt(0,MID ) then
    bwBrowser.Chromium.SendProcessMessage(PID_RENDERER, TempMsg);
end;

procedure TMainForm.AddTrayMenuItem(aAPIID : integer; ACaption : String);

Var
 Itm : TTrayMenuItem;

begin
  Itm:=TTrayMenuItem.Create(Self);
  Itm.APIID:=aAPIID;
  Itm.Caption:=aCaption;
  Itm.OnClick:=@HandleTrayIconClick;
  PMTray.Items.Add(Itm);
end;

procedure TMainForm.RemoveTrayMenuItem(aAPIID : integer);

Var
  Itm : TTrayMenuItem;
  I : Integer;

begin
  Itm:=Nil;
  I:=PMTray.Items.Count;
  While (I>=0) and (Itm=Nil) do
    begin
    Itm:=TTrayMenuItem(PMTray.Items[i]);
    if (Itm.APIID<>aAPIID) then
      Itm:=Nil;
    Dec(I);
    end;
  Itm.Free;
end;

procedure TMainForm.SetTrayIconVisibility(aVisible : PtrInt);

begin
  TIApp.Visible:=aVisible<>0;
end;


procedure TMainForm.FormCreate(Sender: TObject);
begin
  TLocalResourceHandler.BaseDir:=ExtractFilePath(Paramstr(0));
  TLocalResourceHandler.RegisterHandler('local');
  bwBrowser.Chromium.OnProcessMessageReceived:=@HandleProcessMessage;
  bwBrowser.LoadURL(STrayIconURL);
end;

procedure TMainForm.bwBrowserBrowserCreated(Sender: TObject);
begin
  Caption := 'Browser ready';
end;

procedure TMainForm.ShowLogMsg(Data: PtrInt);

begin
  MLog.Lines.Add(FLogMsg);
end;

procedure TMainForm.DoLog(const aMsg: String);
begin
  FLogMsg:=aMsg;
  Application.QueueAsyncCall(@ShowLogMsg,0);
end;

procedure TMainForm.DoLog(const aFmt: String; aArgs: array of const);
begin
  DoLog(Format(aFmt,aArgs))
end;

procedure TMainForm.DoSetHTMLTitle(Data: PtrInt);
begin
  Caption:=FNewTitle;
end;

procedure TMainForm.HandleProcessMessage(Sender: TObject;
  const browser: ICefBrowser; const frame: ICefFrame;
  sourceProcess: TCefProcessId; const aMessage: ICefProcessMessage; out
  Result: Boolean);

Var
 aList : ICEFListValue ;

begin
  aList:=Nil;
  if Assigned(aMessage) then
     aList:=aMessage.ArgumentList;
  if not (Assigned(aList) and (aList.getSize>0)) then
    exit;
  Case aMessage.Name of
    SMsgSetTrayIconVisibility:
      Application.QueueAsyncCall(@SetTrayIconVisibility,Ord(aList.getBool(0)));
    SMsgAddTrayMenuItem:
      if aList.getSize>1 then
        AddTrayMenuItem(aList.getInt(0),utf8Encode(aList.getString(1)));
    SMsgRemoveTrayMenuItem:
      RemoveTrayMenuItem(aList.getInt(0));
    SMsgDisplayLogmessage:
      DoLog('Render process: '+UTF8Encode(aList.getString(0)));
  end;

end;

{$IFDEF WINDOWS}
procedure TMainForm.WMEnterMenuLoop(var aMessage: TMessage);
begin
  inherited;
  if (aMessage.wParam = 0) and (GlobalCEFApp <> nil) then
    GlobalCEFApp.OsmodalLoop := True;
end;

procedure TMainForm.WMExitMenuLoop(var aMessage: TMessage);
begin
  inherited;
  if (aMessage.wParam = 0) and (GlobalCEFApp <> nil) then
    GlobalCEFApp.OsmodalLoop := False;
end;
{$ENDIF}

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  With bwBrowser do
    begin
    CloseBrowser(True);
    CanClose:=IsClosed;
    end;
end;

procedure TMainForm.btnBackClick(Sender: TObject);
begin
  if bwbrowser.Chromium.CanGoBack then
    bwbrowser.Chromium.GoBack;
end;

procedure TMainForm.ChromiumTitleChange(Sender: TObject;
  const browser: ICefBrowser; const title: ustring);
begin
  if (length(title) > 0) then
    FNewTitle := UTF8enCode(title)
   else
    FNewTitle:=UTF8enCode(bwBrowser.Chromium.DocumentURL);
  Application.QueueAsyncCall(@DoSetHTMLTitle,0);
end;

procedure TMainForm.bwBrowserBrowserClosed(Sender: TObject);
begin
  Close;
end;


initialization
  InitCEF;

finalization
  StopCEF;
end.

