unit localresources;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, uCEFTypes, uCEFApplication, uCEFWorkScheduler,  uCEFInterfaces, uCEFResourceHandler;

Type

  { TLocalResourceHandler }

  TLocalResourceHandler = Class(TCefResourceHandlerOwn)
  private
    FFileName : String;
    FStream : TStream;
  Public
    class var BaseDir : String;
    Class procedure RegisterHandler(aScheme : string);
    function  ProcessRequest(const CEFRequest: ICefRequest; const callback: ICefCallback): Boolean; override;
    procedure GetResponseHeaders(const CEFResponse: ICefResponse; out responseLength: Int64; out redirectUrl: ustring); override;
    function  ReadResponse(const dataOut: Pointer; bytesToRead: Integer; var bytesRead: Integer; const callback: ICefCallback): Boolean; override;
    Destructor destroy; override;
  end;


implementation

uses uCEFMiscFunctions,SysUtils;

{ TLocalResourceHandler }

class procedure TLocalResourceHandler.RegisterHandler(aScheme : String);
begin
  CefRegisterSchemeHandlerFactory('local','',Self);
end;

function TLocalResourceHandler.ProcessRequest(const CEFRequest: ICefRequest;
  const callback: ICefCallback): Boolean;

Var
  S : String;
  P : Integer;

begin
  S:=CEFRequest.GetUrl;
  P:=Pos('://',S);
  if P>0 then
    Delete(S,1,P+2);
  FFileName:=SysUtils.SetDirSeparators(BaseDir+S);
  Result:=FileExists(FFileName);
  if Result then
    FStream:=TFileStream.Create(FFileName,fmOpenRead or fmShareDenyWrite);
  if assigned(CallBack) then
    Callback.Cont;
end;

procedure TLocalResourceHandler.GetResponseHeaders(
  const CEFResponse: ICefResponse; out responseLength: Int64; out
  redirectUrl: ustring);

Var
  Ext : String;

begin
  Ext:=ExtractFileExt(FFileName);
  Ext:=Copy(Ext,2,Length(Ext)-1);
  if Assigned(FStream) then
    begin
    CefResponse.Status:=200;
    CefResponse.StatusText:='OK';
    responseLength:=FStream.Size;
    end
  else
    begin
    CefResponse.Status:=400;
    CefResponse.StatusText:='NOT FOUND';
    responseLength:=0;
    end;
  CefResponse.MimeType:=CefGetMimeType(Ext);
  // No other headers.
end;

function TLocalResourceHandler.ReadResponse(const dataOut: Pointer;
  bytesToRead: Integer; var bytesRead: Integer; const callback: ICefCallback
  ): Boolean;


Var
  aAvail : Integer;

begin
  Result:=Assigned(FStream) and Assigned(DataOut);
  if Result then
    begin
    aAvail:=FStream.Size-FStream.Position;
    If aAvail>BytesToRead then
      aAvail:=BytesToRead;
    BytesRead:=FStream.Read(DataOut^,aAvail);
    Result:=aAvail>0;
    end;
end;

destructor TLocalResourceHandler.destroy;
begin
  FreeAndNil(FStream);
  inherited destroy;
end;



end.

